                   (* +-------------------------+
                      |  tude de permutations  |
                      +-------------------------+ *)


(* On reprsente une permutation f par le tableau t tq t.(i) = f(i) *)

(* 1. Vrifier qu'un tableau reprsente bien une permutation *)
let vrif(t) =
  let n  = vect_length(t)    in
  let u  = make_vect n false in
  let ok = ref(true)
  and i  = ref(0)            in
  while !ok && !i < n do
    if (t.(!i) < 0) or (t.(!i) >= n) or u.(t.(!i)) then ok := false
    else u.(t.(!i)) <- true;
    i := !i + 1;
  done;
  !ok
;;

vrif [| 0; -3; 3; 4; 2 |];;
vrif [| 0; 3.5; 3; 4; 2 |];;
vrif [| 0; 6; 3; 4; 2 |];;
vrif [| 0; 2; 3; 4; 2 |];;
vrif [| 0; 1; 3; 4; 2 |];;

(* 2. Calculer l'orbite d'un lment *)
let orbite t x =
  let p = vect_length(t) in
  let u = make_vect p 0  in
  let n = ref(0)         in
  u.(0) <- x;
  let y = ref(t.(x))     in
  while !y <> x do
    n := !n + 1;
    u.(!n) <- !y;
    y := t.(!y)
  done;
  sub_vect u 0 (!n+1)
;;

let t = [| 0; 4; 1; 3; 2 |];;
orbite t 1;;

(* 3. Dcomposition en cycles *)
let cycles(t) =
  let n = vect_length(t)    in
  let u = make_vect n false in (* lts dja vus *)
  let r = make_vect n [||]  in (* rsultat      *)
  let i = ref(0)            in (* indice pour r *)
  let encore = ref(true)   in
  while !encore do
    let j = ref(0) in while (!j < n) && u.(!j) do j := !j+1 done;
    if !j >= n then encore := false
    else begin
      let o = orbite t !j in
      r.(!i) <- o;
      i := !i+1;
      for k=0 to vect_length(o)-1 do u.(o.(k)) <- true done;
    end;
  done;
  sub_vect r 0 !i
;;

t;;
cycles(t);;

(* 4. Signature et ordre *)
let signature(t) =
  let c = cycles(t) in
  let s = ref(1)    in
  for i=0 to vect_length(c)-1 do
    if vect_length(c.(i)) mod 2 = 0 then s := - !s
  done;
  !s
;;

signature(t);;
signature[|1; 2; 3; 4; 5; 0|];;

let rec pgcd a b = if b=0 then a else pgcd b (a mod b);;
let ppcm a b = (a*b)/(pgcd a b);;

let ordre(t) =
  let c = cycles(t) in
  let p = ref(1)    in
  for i=0 to vect_length(c)-1 do p := ppcm !p (vect_length c.(i)) done;
  !p
;;

ordre t;;
ordre [| 1; 0; 3; 2; 5; 6; 4 |];;
